import java.awt.*;
import java.awt.event.*;
import java.applet.*;


class Point extends Object{
  public int x,y;
  public Point(){}
  public Point(Point p){x=p.x;y=p.y;}
}
public class BezierDraw extends Applet
  implements MouseListener, MouseMotionListener,ActionListener{

  Point pts[],oldp[];
  int points;
  Font f;
  int xpos,ypos;
  int radius=3;
  int diameter=radius*2;
  boolean mouse_dragging;
  int cur_point;

  public void paint(Graphics g){

    g.setColor(Color.darkGray);
    g.drawString("(c)Ken Power, I.T.Carlow 2001",0,10);
    g.setFont(f);
    g.setColor(Color.black);
    for(int i=0;i<points;i++){
      g.drawOval((pts[i]).x-radius,(pts[i]).y-radius,diameter,diameter);
      g.drawString("P"+i,(pts[i]).x+radius,(pts[i]).y-diameter);
    }
//    g.setColor(Color.yellow);

    g.setColor(Color.black);

    g.setColor(Color.lightGray);
    if(cur_point>-1){
      drawCurve(g,oldp);
    }
    g.setColor(Color.black);
    drawCurve(g,pts);

  }

  void drawCurve(Graphics g,Point[] p){
    if(points>1){
      double qx,qy,lastx=(double)(p[0]).x,lasty=(double)(p[0]).y,b,t;
      int k;
      for(t=0;t<=1;t+=0.01){
        qx=qy=0.0;
        for(k=0;k<points;k++){
          b=blend(k,points-1,t);
          qx+=(double)(p[k]).x*b;
          qy+=(double)(p[k]).y*b;
        }
        g.drawLine((int)lastx,(int)lasty,(int)qx,(int)qy);
        lastx=(int)qx;
        lasty=(int)qy;
      }
      g.drawLine((int)lastx,(int)lasty,(p[points-1]).x,(p[points-1]).y);
    }

  }

  double blend(int k,int L,double t){
    double b;
    b=(fact(L)/(fact(k)*fact(L-k)))*pow((1-t),(L-k))*pow(t,k);
    return b;
  }

  double pow(double a,int b){
    double x=1.0;
    for(int i=0;i<b;i++){
      x*=a;
    }
    return x;
  }

  double fact(int num){
    double f=1;
    for(int i=1;i<=num;i++)
      f*=i;
    return f;
  }

  public void mouseClicked(MouseEvent e){
    Point t=new Point();
    t.x=e.getX();
    t.y=e.getY();
    pts[points++]=t;
    cur_point=-1;
    repaint();

  }
  public void mouseReleased(MouseEvent e){

    cur_point=-1;
    //this.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
    repaint();

  }
  public void mousePressed(MouseEvent e){

    cur_point=-1;
    double cur_dist=10000.0;

    double d;
    for(int i=0;i<points;i++){
      d=dist(pts[i],e.getX(),e.getY());
      if(d<cur_dist){
        cur_dist=d;
        cur_point=i;
      }
    }
    if(cur_dist<diameter){
        for(int j=0;j<points;j++)
          oldp[j]=pts[j];
        oldp[cur_point]=new Point(pts[cur_point]);
        this.setCursor(new Cursor(Cursor.MOVE_CURSOR));
    }
    else{
        cur_point=-1;
    }
  }

  double dist(Point pt,int x, int y){
    return Math.sqrt((pt.x-x)*(pt.x-x)+(pt.y-y)*(pt.y-y));
  }
  void remove_point(Point[]p,int i){
    for(int j=i;j<(points-1);j++)
      p[j]=p[j+1];
    if(points>0) points--;
  }
  public void mouseExited(MouseEvent e){
     if(cur_point>-1){
      remove_point(pts,cur_point);
     }
    cur_point=-1;
    repaint();
   }
  public void mouseEntered(MouseEvent e){}

  public void mouseMoved(MouseEvent e){
    //cur_point=-1;
    double cur_dist=10000.0;

    double d;
    for(int i=0;i<points;i++){
      d=dist(pts[i],e.getX(),e.getY());
      if(d<cur_dist){
        cur_dist=d;
        //cur_point=i;
      }
    }
    if(cur_dist<diameter){
      this.setCursor(new Cursor(Cursor.MOVE_CURSOR));
    }
    else{
          this.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
          //cur_point=-1;
      }

  }
  public void mouseDragged(MouseEvent e){
    if(cur_point>-1){
      Point pt=pts[cur_point];
      pt.x=e.getX();
      pt.y=e.getY();
      repaint();
    }
  }

  public void init() {
    setSize(new Dimension(400,300));
    setLayout(new BorderLayout());

    Button clear_btn=new Button("Clear");
    add(clear_btn,"South");
    clear_btn.addActionListener(this);
    setBackground(Color.yellow);

    addMouseListener(this);
    addMouseMotionListener(this);
    pts=new Point[1000];
    oldp=new Point[1000];
    cur_point=-1;
    points=0;
    f=new Font("Arial",Font.PLAIN,10);
  }
  public void actionPerformed(ActionEvent e){
      pts=new Point[1000];
      points=0;
      repaint();
      cur_point=-1;

  }

}
